<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Api\Controller;

use Dedi\SyliusVoucherPlugin\Api\Enum\ErrorMessage;
use Dedi\SyliusVoucherPlugin\Applicator\VoucherApplicatorInterface;
use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Sylius\Component\Core\Model\ShopUserInterface;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class ApplyVoucherAction extends AbstractController
{
    public function __construct(
        private readonly OrderRepositoryInterface $orderRepository,
        private readonly TokenStorageInterface $tokenStorage,
        private readonly VoucherRepositoryInterface $voucherRepository,
        private readonly VoucherApplicatorInterface $voucherApplicator,
    ) {
    }

    public function __invoke(int $id, string $voucherCode): JsonResponse
    {
        $token = $this->tokenStorage->getToken();
        $user = $token?->getUser();

        if (!$user instanceof ShopUserInterface) {
            return new JsonResponse(status: Response::HTTP_UNAUTHORIZED);
        }

        $order = $this->orderRepository->findCartById($id);

        if (!$order instanceof OrderInterface) {
            return new JsonResponse(ErrorMessage::ORDER_NOT_FOUND->value, Response::HTTP_NOT_FOUND);
        }

        if ($order->getUser() !== $user) {
            return new JsonResponse(status: Response::HTTP_FORBIDDEN);
        }

        $voucher = $this->voucherRepository->findEnabledNotExpiredByCodeAndUserAndChannel(
            $voucherCode,
            $user->getCustomer(),
            $order->getChannel(),
        );

        if (!$voucher instanceof VoucherInterface) {
            return new JsonResponse(ErrorMessage::VOUCHER_NOT_FOUND->value, Response::HTTP_NOT_FOUND);
        }

        if ($order->hasVoucher($voucher)) {
            return new JsonResponse(ErrorMessage::VOUCHER_ALREADY_APPLIED->value, Response::HTTP_UNPROCESSABLE_ENTITY);
        }

        try {
            $this->voucherApplicator->apply($order, $voucher);
        } catch (\Exception $exception) {
            return new JsonResponse($exception->getMessage(), Response::HTTP_INTERNAL_SERVER_ERROR);
        }

        return new JsonResponse(status: Response::HTTP_NO_CONTENT);
    }
}
