<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Form\Type;

use Dedi\SyliusVoucherPlugin\Generator\VoucherCodeGeneratorInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Sylius\Bundle\ResourceBundle\Form\EventSubscriber\AddCodeFormSubscriber;
use Sylius\Bundle\ResourceBundle\Form\Type\AbstractResourceType;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\Form\CallbackTransformer;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;

final class VoucherApiType extends AbstractResourceType
{
    private readonly RepositoryInterface $currencyRepository;

    private readonly VoucherCodeGeneratorInterface $voucherCodeGenerator;

    private readonly RepositoryInterface $channelRepository;

    public function __construct(
        string $dataClass,
        RepositoryInterface $currencyRepository,
        RepositoryInterface $channelRepository,
        VoucherCodeGeneratorInterface $voucherCodeGenerator,
        array $validationGroups = [],
    ) {
        parent::__construct($dataClass, $validationGroups);

        $this->currencyRepository = $currencyRepository;
        $this->voucherCodeGenerator = $voucherCodeGenerator;
        $this->channelRepository = $channelRepository;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->addEventSubscriber(new AddCodeFormSubscriber())
            ->add('customer', CustomerAutocompleteChoiceType::class)
            ->add('initialAmount', NumberType::class, [
                'label' => 'sylius.ui.amount',
            ])
            ->add('enabled', CheckboxType::class, [
                'label' => 'sylius.ui.enabled',
                'required' => false,
            ])
            ->add('dateEnd', DateType::class, [
                'label' => 'dedi_sylius_voucher.ui.date_end',
                'required' => false,
                'html5' => true,
                'widget' => 'single_text',
            ])
            ->addEventListener(FormEvents::PRE_SET_DATA, function (FormEvent $event): void {
                /** @var VoucherInterface $voucher */
                $voucher = $event->getData();

                if ($voucher->getCode() === null) {
                    $voucher->setCode($this->voucherCodeGenerator->generate());
                }

                $form = $event->getForm();
                $form
                    ->add('currencyCode', ChoiceType::class, [
                        'label' => 'sylius.ui.currency',
                        'choices' => $this->currencyRepository->findAll(),
                        'choice_label' => 'code',
                        'choice_value' => 'code',
                    ])
                ;

                $form
                    ->add('channel', ChoiceType::class, [
                        'label' => 'sylius.ui.channel',
                        'choices' => $this->channelRepository->findAll(),
                        'choice_label' => 'code',
                        'choice_value' => 'code',
                    ])
                ;
            })
        ;

        $builder->get('initialAmount')->addModelTransformer(new CallbackTransformer(static function (?int $amount): ?float {
            if (null === $amount) {
                return null;
            }

            return round($amount / 100, 2);
        }, static function (?float $amount): ?int {
            if (null === $amount) {
                return null;
            }

            return (int) round($amount * 100);
        }));
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_sylius_voucher_voucher';
    }
}
